#pragma once

#include "stdafx.h"
#include <string>
#include <vector>
#include <list>

typedef std::pair<std::wstring, std::wstring> Value;
typedef std::list<Value> Values;
typedef std::pair<std::wstring, Values> Section;
typedef std::list<Section> Sections;

class CIniFile
{
public:
	CIniFile(const HINSTANCE hInstance, const BOOL IsLoad = TRUE);
	CIniFile(const std::wstring IniFileName, const BOOL IsLoad = TRUE);
	~CIniFile(void);

	BOOL Load(void);
	BOOL Save(void);

	BOOL Fail(void) { return m_Fail; };

	Sections &GetSections(void);
	Values &GetValues(const std::wstring &SectionName, BOOL IsCreate = FALSE);

	std::wstring &GetValue(const std::wstring &SectionName, const std::wstring &KeyName);
	std::wstring &GetValue(const std::wstring &SectionName, const std::wstring &KeyName, std::wstring &DefaultValue, BOOL IsSave = TRUE);
	int GetIntValue(const std::wstring &SectionName, const std::wstring &KeyName);
	int GetIntValue(const std::wstring &SectionName, const std::wstring &KeyName, int DefaultValue, BOOL IsSave = TRUE);

	BOOL SetValue(const std::wstring &SectionName, const std::wstring &KeyName, const std::wstring &ValueData, BOOL WhenNotFoundOnly = FALSE);
	BOOL SetIntValue(const std::wstring &SectionName, const std::wstring &KeyName, const int ValueData, BOOL WhenNotFoundOnly = FALSE);

	void AddValue(const std::wstring &SectionName, const std::wstring &KeyName, const std::wstring &ValueData);
	void AddIntValue(const std::wstring &SectionName, const std::wstring &KeyName, const int ValueData);

	BOOL DeleteSection(const std::wstring &SectionName);
	BOOL DeleteValues(const std::wstring &SectionName);
	BOOL DeleteValue(const std::wstring &SectionName, const std::wstring &KeyName);

	std::wstring &GetIniFileName(void);
private:
	typedef std::vector<std::wstring> Strings;

	void Release(void);
	std::wstring GetDefaultIniFileName(const HINSTANCE hInstance);
	Strings ReadIniFile(void);
	BOOL WriteIniFile(std::wstring data);
	Sections::iterator GetSectionPair(const std::wstring &SectionName);
	Values::iterator GetValuePair(Values &values, const std::wstring &KeyName);

private:
	std::wstring m_IniFileName;
	Sections m_Sections;
	BOOL m_Fail;

};
